@@ -18,19 +18,31 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 -}
 {-# LANGUAGE CPP #-}
 module Bustle.Diagram
-  ( Shape(..)
-  , Diagram
+  ( Diagram
+
+  -- Shapes, and smart constructors therefore
+  , Shape(..)
+  , memberLabel
+  , timestampLabel
+  , headers
+  , headerHeight
+
+  -- Attributes of shapes
   , Arrowhead(..)
   , Side(..)
   , Colour(..)
   , Rect
-  , dimensions
-  , topJustify
+
+  -- Annoying constants that users of this module need.
+  , columnWidth
+  , timestampAndMemberWidth
+  , firstColumnOffset
+
+  -- Displaying diagrams
+  , diagramDimensions
+  , topLeftJustifyDiagram
   , drawDiagram
   , drawRegion
-  , headers
-  , headerHeight
-  , columnWidth
   )
 where
 
@@ -81,11 +93,15 @@ data Shape = Header { strs :: [String]
                     , shapex, shapey :: Double
                     }
            | MemberLabel { labelPath, labelMember :: String
-                         , shapey :: Double
+                         , shapex :: Double -- The coordinates of the *centre*
+                         , shapey :: Double -- of the label
                          }
-           | Timestamp { str :: String, shapey :: Double }
+           | TimestampLabel { str :: String
+                            , shapex :: Double -- The coordinates of the
+                            , shapey :: Double -- *centre* of the timestamp
+                            }
            | ClientLine { shapex, shapey1, shapey2 :: Double }
-           | Rule { shapex, shapey :: Double }
+           | Rule { shapex1, shapex2, shapey :: Double }
            | Arrow { shapecolour :: Maybe Colour
                    , arrowhead :: Arrowhead
                    , shapex1, shapex2, shapey :: Double
@@ -97,6 +113,14 @@ data Shape = Header { strs :: [String]
                  }
   deriving (Show, Read, Eq)
 
+-- Smart constructors for TimestampLabel and MemberLabel that fill in the
+-- hardcoded (spit) x coordinates.
+memberLabel :: String -> String -> Double -> Shape
+memberLabel p m y = MemberLabel p m memberx y
+
+timestampLabel :: String -> Double -> Shape
+timestampLabel s y = TimestampLabel s timestampx y
+
 type Diagram = [Shape]
 
 arcControlPoints :: Shape -> (Point, Point)
@@ -107,7 +131,23 @@ arcControlPoints (Arc { topx=x1, topy=y1, bottomx=x2, bottomy=y2, arcside=s }) =
     in (cp1, cp2)
 arcControlPoints _ = error "i see you've played arcy-shapey before"
 
-mapY :: (Double -> Double) -> (Shape -> Shape)
+mapX, mapY :: (Double -> Double) -> (Shape -> Shape)
+mapX f s = case s of
+    Rule {}        -> s { shapex1 = f (shapex1 s)
+                        , shapex2 = f (shapex2 s)
+                        }
+    Arrow {}       -> s { shapex1 = f (shapex1 s)
+                        , shapex2 = f (shapex2 s)
+                        }
+    SignalArrow {} -> s { shapex1 = f (shapex1 s)
+                        , epicentre = f (epicentre s)
+                        , shapex2 = f (shapex2 s)
+                        }
+    Arc {}         -> s { topx = f (topx s)
+                        , bottomx = f (bottomx s)
+                        }
+    _              -> s { shapex = f (shapex s) }
+
 mapY f s = case s of
     Arc {}        -> s { topy = f (topy s)
                        , bottomy = f (bottomy s)
@@ -124,16 +164,24 @@ eventHeight :: Double
 eventHeight = 30
 
 timestampx, timestampWidth :: Double
-timestampx = 30
+timestampx = 0 + timestampWidth / 2
 timestampWidth = 60
 
 memberx, memberWidth :: Double
-memberx = 230
+memberx = timestampWidth + memberWidth / 2
 memberWidth = 340
 
+timestampAndMemberWidth :: Double
+timestampAndMemberWidth = timestampWidth + memberWidth
+
 columnWidth :: Double
 columnWidth = 90
 
+-- Method return arcs can go outside the first column. Empirically, 20 is
+-- enough to stop the arc (or the duration text) overlapping the object path
+-- etc.
+firstColumnOffset :: Double
+firstColumnOffset = 20 + columnWidth / 2
 
 --
 -- Calculating bounds of shapes
@@ -156,7 +204,7 @@ headerHeight = fromIntegral . (10 *) . length
 bounds :: Shape -> Rect
 bounds s = case s of
   ClientLine {} -> (shapex s, shapey1 s, shapex s, shapey2 s)
-  Rule {} -> (0, shapey s, shapex s, shapey s)
+  Rule {} -> (shapex1 s, shapey s, shapex2 s, shapey s)
   Arrow {} ->
     let (x1, x2) = xMinMax s
         y1 = shapey s - (if above (arrowhead s) then 5 else 0)
@@ -170,8 +218,8 @@ bounds s = case s of
     let ((cx, _), (dx, _)) = arcControlPoints s
        -- FIXME: magic 5 makes the bounding box include the text
     in (min x1 cx, y1, max x2 dx, y2 + 5)
-  Timestamp { shapey=y } -> fromCentre timestampx y timestampWidth
-  MemberLabel _ _ y -> fromCentre memberx y memberWidth
+  TimestampLabel { shapex=x, shapey=y } -> fromCentre x y timestampWidth
+  MemberLabel _ _ x y -> fromCentre x y memberWidth
   Header { strs = ss, shapex = x, shapey = y} ->
     let width = columnWidth
         height = headerHeight ss
@@ -198,34 +246,51 @@ headers xss y = (height, shapes)
 -- Drawing
 --
 
-dimensions :: Diagram -> (Double, Double)
-dimensions shapes = (x2, y2)
-    where (_, _, x2, y2) = corners shapes
+diagramBounds :: Diagram -> ((Double, Double), (Double, Double))
+diagramBounds shapes = ((minimum (0:x1s), minimum (0:y1s))
+                       ,(maximum (0:x2s), maximum (0:y2s))
+                       )
+  where
+    (x1s, y1s, x2s, y2s) = unzip4 $ map bounds shapes
 
-corners :: Diagram -> (Double, Double, Double, Double)
-corners [] = (0, 0, 0, 0)
-corners shapes = (minimum x1s, minimum y1s,
-                  maximum x2s, maximum y2s)
-    where (x1s, y1s, x2s, y2s) = unzip4 $ map bounds shapes
+diagramDimensions :: Diagram -> (Double, Double)
+diagramDimensions shapes = (x2 - x1, y2 - y1)
+  where
+    ((x1, y1), (x2, y2)) = diagramBounds shapes
 
-topJustify :: Diagram -> Diagram
-topJustify shapes = f shapes
-    where (_, y1, _, _) = corners shapes
-          f             = if y1 < 0 then map (mapY (subtract y1)) else id
+topLeftJustifyDiagram :: Diagram -- ^ the original diagram
+                      -> (Double, Diagram) -- ^ the diagram transformed to be
+                                           --   in positive space, and the
+                                           --   x-axis shift necessary to do so
+topLeftJustifyDiagram shapes = (negate x1, shapes')
+  where
+    shapes'       = transformDiagram (negate x1, negate y1) shapes
+    ((x1, y1), _) = diagramBounds shapes
 
-drawDiagram :: Bool -> Diagram -> Render ()
-drawDiagram drawBounds shapes = do
+transformDiagram :: (Double, Double) -> (Diagram -> Diagram)
+transformDiagram (x, y) = map (mapX (+ x) . mapY (+ y))
+
+drawDiagramInternal :: (Shape -> Bool) -- ^ A filter for the shapes
+                    -> Bool -- ^ True to draw canvas items' bounding boxes
+                            --   (for debugging)
+                    -> Diagram   -- ^ A diagram to render
+                    -> Render ()
+drawDiagramInternal f drawBounds shapes = do
     clearCanvas
-    forM_ shapes $ \x -> do
+
+    forM_ (filter f shapes) $ \x -> do
         when drawBounds (drawBoundingBox x)
         draw x
 
+drawDiagram :: Bool      -- ^ True to draw canvas items' bounding boxes (for
+                         --   debugging)
+            -> Diagram   -- ^ A diagram to render
+            -> Render ()
+drawDiagram = drawDiagramInternal (const True)
+
 drawRegion :: Rect -> Bool -> Diagram -> Render ()
-drawRegion r db = drawDiagram db . visible r . map (bounds &&& id)
-
-visible :: Rect -> [(Rect, Shape)] -> [Shape]
-visible r = map snd . filter (intersects r . fst)
-
+drawRegion r = drawDiagramInternal isVisible
+    where isVisible = intersects r . bounds
 
 saved :: Render () -> Render ()
 saved act = save >> act >> restore
@@ -254,10 +319,17 @@ draw s = draw' s
           Arrow {} -> drawArrow <$> shapecolour <*> arrowhead <*> shapex1 <*>
                         shapex2 <*> shapey
           Header {} -> drawHeader <$> strs <*> shapex <*> shapey
-          MemberLabel s1 s2 y -> const (drawMember s1 s2 y)
-          Timestamp {} -> drawTimestamp <$> str <*> shapey
+          MemberLabel {} -> drawMember <$> labelPath
+                                       <*> labelMember
+                                       <*> shapex
+                                       <*> shapey
+          TimestampLabel {} -> drawTimestamp <$> str
+                                             <*> shapex
+                                             <*> shapey
           ClientLine {} -> drawClientLine <$> shapex <*> shapey1 <*> shapey2
-          Rule {} -> drawRule <$> shapex <*> shapey
+          Rule {} -> drawRule <$> shapex1
+                              <*> shapex2
+                              <*> shapey
 
 
 halfArrowHead :: Arrowhead -> Bool -> Render ()
@@ -349,17 +421,17 @@ drawHeader names x y = forM_ (zip [0..] names) $ \(i, name) -> do
     showLayout l
   where h = 10
 
-drawMember :: String -> String -> Double -> Render ()
-drawMember s1 s2 y = dm s1 (y - 10) >> dm s2 y
+drawMember :: String -> String -> Double -> Double -> Render ()
+drawMember s1 s2 x y = dm s1 (y - 10) >> dm s2 y
   where
     dm s y' = do
       l <- mkLayout s EllipsizeStart AlignLeft `withWidth` memberWidth
-      moveTo timestampWidth y'
+      moveTo (x - memberWidth / 2) y'
       showLayout l
 
-drawTimestamp :: String -> Double -> Render ()
-drawTimestamp ts y = do
-    moveTo 0 (y - 10)
+drawTimestamp :: String -> Double -> Double -> Render ()
+drawTimestamp ts x y = do
+    moveTo (x - timestampWidth / 2) (y - 10)
     showLayout =<< mkLayout ts EllipsizeNone AlignLeft `withWidth` timestampWidth
 
 drawClientLine :: Double -> Double -> Double -> Render ()
@@ -369,13 +441,13 @@ drawClientLine x y1 y2 = saved $ do
     lineTo x y2
     stroke
 
-drawRule :: Double -> Double -> Render ()
-drawRule x y = saved $ do
-    setSourceRGB 0.7 0.1 0.1
-    setLineWidth 0.2
+drawRule :: Double -> Double -> Double -> Render ()
+drawRule x1 x2 y = saved $ do
+    setSourceRGB 0.9 0.9 0.9
+    setLineWidth 0.5
 
-    moveTo 0 y
-    lineTo x y
+    moveTo x1 y
+    lineTo x2 y
     stroke
 
 -- vim: sw=2 sts=2
